package com.engine.salary.mapper.setting;

import com.engine.salary.entity.setting.po.PageListSettingPO;
import org.apache.ibatis.annotations.Param;

import java.util.Collection;
import java.util.List;

public interface PageListSettingMapper {

	/**
     * 查询所有记录
     *
     * @return 返回集合，没有返回空List
     */
	List<PageListSettingPO> listAll();
    
    /**
     * 条件查询
     *
     * @return 返回集合，没有返回空List
     */
    List<PageListSettingPO> listSome(PageListSettingPO pageListSetting);
    

	/**
     * 根据主键查询
     *
     * @param id 主键
     * @return 返回记录，没有返回null
     */
	PageListSettingPO getById(Long id);
	
	/**
     * 新增，忽略null字段
     *
     * @param pageListSetting 新增的记录
     * @return 返回影响行数
     */
	int insertIgnoreNull(PageListSettingPO pageListSetting);
	
	/**
     * 修改，修改所有字段
     *
     * @param pageListSetting 修改的记录
     * @return 返回影响行数
     */
	int update(PageListSettingPO pageListSetting);
	
	/**
     * 修改，忽略null字段
     *
     * @param pageListSetting 修改的记录
     * @return 返回影响行数
     */
	int updateIgnoreNull(PageListSettingPO pageListSetting);
	
	/**
     * 删除记录
     *
     * @param pageListSetting 待删除的记录
     * @return 返回影响行数
     */
	int delete(PageListSettingPO pageListSetting);
    
    /**
     * 批量删除记录
     * @param ids 主键id集合
     */
    void deleteByIds(@Param("ids") Collection<Long> ids);

	//获取页面默认配置
	PageListSettingPO getByPage(String page);

	//获取页面配置模板
	List<PageListSettingPO> getTemplatesByPage(String page);
}